@@ -4,32 +4,46 @@ from enum import Enum
 from geomet import wkt
 
 
+def format_date(date):
+    try:
+        return isoparse(date)
+    except ValueError:
+        raise ValueError('Datetime format not recognised')
+
+
 class EDRQueryParser:
+    QUERY_TYPES = Enum('query_type', 'position radius area cube trajectory corridor items locations')
+
     def __init__(self, url):
-        self.url_list = list(filter(None, urlsplit(url).path.split('/')))
-        self.query_dic = {str(key): ''.join(value) for key, value in parse_qs(urlsplit(url).query).items()}
+        self.url_parts = list(filter(None, urlsplit(url).path.split('/')))
+        self.query_parts = {str(key): ''.join(value) for key, value in parse_qs(urlsplit(url).query).items()}
 
     def _get_id(self, query_id):
-        if self.query_type == query_id:
-            query_index = self.url_list.index(query_id)
-            if len(self.url_list) > query_index + 1:
-                return self.url_list[query_index + 1]
-        return None
+        try:
+            return self.url_parts[self.url_parts.index(query_id) + 1]
+        except IndexError:
+            return None
 
     @property
     def collection_name(self):
-        if len(self.url_list) != self.url_list.index('collections') + 2:
-            return self.url_list[self.url_list.index('collections') + 1]
-        raise ValueError('collection name not found in url')
+        try:
+            return self.url_parts[self.url_parts.index('collections') + 1]
+        except (ValueError, IndexError):
+            raise ValueError('collection name not found in url')
 
     @property
     def query_type(self):
-        query_type = Enum('query_type', 'position radius area cube trajectory corridor items locations instances')
         try:
-            return query_type[self.url_list[self.url_list.index('collections') + 2]].name
+            if self.is_instances:
+                return self.QUERY_TYPES[self.url_parts[-1]].name
+            return self.QUERY_TYPES[self.url_parts[self.url_parts.index('collections') + 2]].name
         except KeyError:
             raise ValueError('unsupported query type found in url')
 
+    @property
+    def is_instances(self):
+        return self.url_parts[self.url_parts.index('collections') + 2] == 'instances'
+
     @property
     def items_id(self):
         return self._get_id('items')
@@ -44,27 +58,31 @@ class EDRQueryParser:
 
     @property
     def format(self):
-        return Parameter(self.query_dic.get('f'))
+        return Parameter(self.query_parts.get('f'))
 
     @property
     def coords(self):
-        return Coords(self.query_dic.get('coords'))
+        return Coords(self.query_parts.get('coords'))
 
     @property
     def crs(self):
-        return Parameter(self.query_dic.get('crs'))
+        return Parameter(self.query_parts.get('crs'))
 
     @property
     def parameter_name(self):
-        return ParameterWithList(self.query_dic.get('parameter-name'))
+        return ParameterWithList(self.query_parts.get('parameter-name'))
 
     @property
     def datetime(self):
-        return DateTime(self.query_dic.get('datetime'))
+        return DateTime(self.query_parts.get('datetime'))
 
     @property
     def z(self):
-        return Z(self.query_dic.get('z'))
+        return Z(self.query_parts.get('z'))
+
+    @property
+    def bbox(self):
+        return ParameterWithFloatList(self.query_parts.get('bbox'))
 
 
 class Parameter:
@@ -89,6 +107,15 @@ class ParameterWithList(Parameter):
         return self.is_set and ',' in self.value
 
 
+class ParameterWithFloatList(ParameterWithList):
+    @property
+    def list(self):
+        try:
+            return list(map(float, super().list))
+        except ValueError:
+            raise ValueError('could not convert parameter to a list')
+
+
 class ParameterWithInterval(Parameter):
     def _split(self, index=None):
         if self.is_set and '/' in self.value:
@@ -117,27 +144,18 @@ class ParameterWithInterval(Parameter):
 class DateTime(ParameterWithInterval):
     @property
     def interval_from(self):
-        try:
-            return isoparse(super().interval_from)
-        except ValueError:
-            raise ValueError('Datetime format not recognised')
+        return format_date(super().interval_from)
 
     @property
     def interval_to(self):
-        try:
-            return isoparse(super().interval_to)
-        except ValueError:
-            raise ValueError('Datetime format not recognised')
+        return format_date(super().interval_to)
 
     @property
     def exact(self):
-        try:
-            return isoparse(self.value)
-        except ValueError:
-            raise ValueError('Datetime format not recognised')
+        return format_date(self.value)
 
 
-class Z(ParameterWithList, ParameterWithInterval):
+class Z(ParameterWithFloatList, ParameterWithInterval):
     @property
     def float(self):
         try:
@@ -163,13 +181,6 @@ class Z(ParameterWithList, ParameterWithInterval):
     def is_all(self):
         return self.is_set and self.value.lower() == 'all'
 
-    @property
-    def list(self):
-        try:
-            return list(map(float, super().list))
-        except ValueError:
-            raise ValueError('could not convert parameter to a list')
-
 
 class Coords(Parameter):
     @property
